@@ -20,15 +20,14 @@ module Pix
 
   def self.surface_to_region(surface)
     region = Cairo::Region.new()
-    data = surface.data
     width = surface.width
-    pix_na = NArray.to_na(data, NArray::BYTE)
+    pix_na = NArray.to_na(surface.data, NArray::BYTE)
     pix_na.reshape!(4, pix_na.size / 4)
     alpha = pix_na[3, true]
     curr_y = -1
     start_x = -1
     end_x = -1
-    (alpha.ne 0).where.each {|i|
+    alpha.where.each {|i|
       y = (i / width)
       x = (i % width)
       if start_x != -1
@@ -91,11 +90,13 @@ module Pix
       set_app_paintable(true)
       set_focus_on_map(false)
       @__surface_position = [0, 0]
+      @prev_position = [0, 0]
       # create drawing area
       @darea = Gtk::DrawingArea.new
       @darea.set_size_request(*size) # XXX
       @darea.show()
       add(@darea)
+      @region = nil
     end
 
     def move(x, y)
@@ -131,15 +132,21 @@ module Pix
       cr.restore()
     end
 
-    def set_shape(cr)
+    def set_shape(cr, reshape)
       return if RbConfig::CONFIG['host_os'] =~ /mswin|mingw|cygwin/
-      region = Pix.surface_to_region(cr.target.map_to_image)
+      if @region.nil? or reshape
+        @region = Pix.surface_to_region(cr.target.map_to_image)
+      else
+        dx, dy = @__surface_position.zip(@prev_position).map {|new, prev| new - prev}
+        @region.translate!(dx, dy)
+      end
+      @prev_position = @__surface_position
       if @supports_alpha
         input_shape_combine_region(nil)
-        input_shape_combine_region(region)
+        input_shape_combine_region(@region)
       else
         shape_combine_region(nil)
-        shape_combine_region(region)
+        shape_combine_region(@region)
       end
     end
   end
