@@ -911,6 +911,7 @@ module Surface
       @y_root = nil
       @click_count = 0
       @__balloon_offset = nil
+      @reshape = true
       @window.signal_connect('leave_notify_event') do |w, e|
         next window_leave_notify(w, e) # XXX
       end
@@ -973,6 +974,7 @@ module Surface
 
     def display_changed(screen)
       return unless @side.zero?
+      @reshape = true # XXX
       @parent.handle_request('NOTIFY', 'reset_position') # XXX
       left, top, scrn_w, scrn_h = @window.workarea
       @parent.handle_request(
@@ -1097,6 +1099,7 @@ module Surface
       else
         @surface_id = surface_id
       end
+      @reshape = true
       @seriko.reset(self, surface_id)
       # define collision areas
       @collisions = @region[@surface_id]
@@ -1304,6 +1307,7 @@ module Surface
 
     def update_frame_buffer
       return if @parent.handle_request('GET', 'lock_repaint')
+      @reshape = true # FIXME: depends on Seriko
       new_surface = create_image_surface(@seriko.get_base_id)
       fail "assert" if new_surface.nil?
       # update collision areas
@@ -1345,7 +1349,8 @@ module Surface
       unless @parent.handle_request('GET', 'get_preference', 'check_collision').zero?
         draw_region(cr)
       end
-      @window.set_shape(cr)
+      @window.set_shape(cr, @reshape)
+      @reshape = false
     end
 
     def remove_overlay(actor)
@@ -1532,6 +1537,7 @@ module Surface
     def show
       return if @parent.handle_request('GET', 'lock_repaint')
       return if @__shown
+      @reshape = true
       @__shown = true
       x, y = get_position()
       @window.move(x, y) # XXX: call before showing the window
